import base64
import json
import requests
from requests.structures import CaseInsensitiveDict

class Ovl:
  def __init__(self, baseUrl):
    self.setUrl(baseUrl)

  def setUrl(self, baseUrl):
    self.baseUrl = baseUrl + "/odata"
    self.authToken = ""

  def login(self, userName, password):
    self.authToken = ""
    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"

    credentials = "{}:{}".format(userName, password)
    headers["Authorization"] = "Basic " + base64.b64encode(credentials.encode()).decode()
    headers["Content-Type"] = "application/json"
    requestBody = {
      "action": "login"
    }

    data = json.dumps(requestBody)

    resp = requests.post(self.baseUrl + "/login", headers=headers, data=data)

    if resp.status_code == 201:
      body = resp.content.decode()

      if len(body) > 0:
        self.authToken = json.loads(body)["value"]["token"]
        return True
    return False

  def isLoggedIn(self):
    if len(self.authToken) > 0:
      return True
    else:
      return False

  def getStatusInfo(self):
    
    result = {"succeeded": False}
    if not self.isLoggedIn():
      return result

    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    headers["Content-Type"] = "application/json"
    headers["Authorization"] = "Bearer " + self.authToken

    params = {"$filter": "keys eq '1'", "config_id":1}

    resp = requests.get(self.baseUrl+"/loggingNodes", headers=headers, params = params)

    if resp.status_code == 200:
      body = resp.content.decode()
      response = json.loads(body)
      if len(response["d"]["results"]) == 1:
        result["succeeded"] = True
        result["badVariables"] = response["d"]["results"][0]["totalBadVariables"]
        result["goodVariables"] = response["d"]["results"][0]["totalGoodVariables"]

      if len(response["d"]["results"][0]["serverStatuses"]) > 0:
        result["serverStatuses"] = []
        serverStatuses = response["d"]["results"][0]["serverStatuses"]
        for serverStatus in serverStatuses:
          sso = {}
          sso["id"] = serverStatus["id"]
          sso["state"] = serverStatus["state"]
          if sso["state"] == 3:
            sso["isConnected"] = True
          else:
            sso["isConnected"] = False
          sso["status"] = serverStatus["status"]
          
          result["serverStatuses"].append(sso)

    return result

  def writeValue(self, variableId, value):
    result = {"succeeded": False}
    if not self.isLoggedIn():
      return result

    # Prepare request headers:
    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    headers["Content-Type"] = "application/json"
    headers["Authorization"] = "Bearer " + self.authToken
    
    # Payload:
    requestBody = {}

    # The value must be string type: 
    requestBody["value"] = "{}".format(value)
    data = json.dumps(requestBody)

    resp = requests.post("{}/loggingNodes({})".format(self.baseUrl, variableId), headers=headers, data = data)

    if resp.status_code == 201:
      result["succeeded"] = True

    return result

  def readValue(self, variableId):
    result = {"succeeded": False}
    if not self.isLoggedIn():
      return result

    # Prepare request headers:
    headers = CaseInsensitiveDict()
    headers["Accept"] = "application/json"
    headers["Content-Type"] = "application/json"
    headers["Authorization"] = "Bearer " + self.authToken
    
    params = {}
    params["$filter"] = "keys eq '{}'".format(variableId)
    params["config_id"] = 1

    resp = requests.get("{}/loggingNodes".format(self.baseUrl), headers=headers, params = params)

    if resp.status_code == 200:
      body = resp.content.decode()
      response = json.loads(body)
      if len(response["d"]["results"]) == 1:
        result["succeeded"] = True
        result["value"] = response["d"]["results"][0]["value"]

        # For complex type values, 2 more fields can be set:
        result["is_complex_type"] = response["d"]["results"][0]["is_complex_type"]
        result["expanded_value"] = response["d"]["results"][0]["expanded_value"]
        
        # Expected "0x00000000 Good" for good status
        result["statusString"] = response["d"]["results"][0]["statusString"]

    return result