import tkinter as tk
import tkinter.ttk as ttk
import time
import threading
import datetime
from Ovl import *

def loginClicked(event):
  if ovl.login("admin", "password"):
    lb.insert("end", "Login succeeded!")
  else:
    lb.insert("end", "Login failed!")

  return

def processGetStatusClickedEvent():
  lb.delete("1.0", "end")
  lb.insert("end", "Processing ...\n")

  ovl.setUrl(urlVar.get())
  if not ovl.login(userVar.get(), passwordVar.get()):
    lb.insert("end", "Login attempt failed\n")
    return

  statusInfo = ovl.getStatusInfo()
  if statusInfo["succeeded"]:
    lb.insert("end", "Number of good variables = {}, bad variables = {}\n".format(statusInfo["goodVariables"], statusInfo["badVariables"]))
    for s in statusInfo["serverStatuses"]:
      lb.insert("end", "Status of the server {} is {}\n".format(s["id"], s["status"]))
  else:
    lb.insert("end", "Failed to get status info")

  return

def processWriteValueClickedEvent():
  lb.delete("1.0", "end")
  lb.insert("end", "Processing write value ...\n")
  
  startTime = datetime.datetime.now()
  ovl.setUrl(urlVar.get())
  if not ovl.login(userVar.get(), passwordVar.get()):
    lb.insert("end", "Login attempt failed\n")
    return
  endTime = datetime.datetime.now()
  d = (endTime-startTime)
  elapsedMs = d.total_seconds() * 1000
  lb.insert("end", "Logged in within {} ms\n".format(elapsedMs))

  varId = variableIdVar.get()
  varValue = valueToWriteVar.get()
  lb.insert("end", "Writing value {} to variable with id {} ...\n".format(varValue, varId))

  startTime = datetime.datetime.now()
  statusInfo = ovl.writeValue(varId, varValue)
  endTime = datetime.datetime.now()
  d = (endTime-startTime)
  elapsedMs = d.total_seconds() * 1000 
  lb.insert("end", "Wrote value in {} ms\n".format(elapsedMs))

  if statusInfo["succeeded"]:
    lb.insert("end", "Write succeeded!\n")
  else:
    lb.insert("end", "Failed to write value.\n")

  return

def processReadValueClickedEvent():
  lb.delete("1.0", "end")
  lb.insert("end", "Processing read value ...\n")
  
  startTime = datetime.datetime.now()
  ovl.setUrl(urlVar.get())
  if not ovl.login(userVar.get(), passwordVar.get()):
    lb.insert("end", "Login attempt failed\n")
    return

  endTime = datetime.datetime.now()
  d = (endTime-startTime)
  elapsedMs = d.total_seconds() * 1000
  lb.insert("end", "Logged in within {} ms\n".format(elapsedMs))

  varId = variableIdVar.get()
  lb.insert("end", "Reading value for variable with id {} ...\n".format(varId))

  startTime = datetime.datetime.now()
  readResult = ovl.readValue(varId)
  endTime = datetime.datetime.now()
  d = (endTime-startTime)
  elapsedMs = d.total_seconds() * 1000 
  lb.insert("end", "Read value in {} ms\n".format(elapsedMs))

  if readResult["succeeded"]:
    lb.insert("end", "Read succeeded!\n")
    currentValueVar.set(readResult["value"])
  else:
    lb.insert("end", "Failed to read value.\n")
  return


def getStatusClicked(event):
  t1 = threading.Thread(target=processGetStatusClickedEvent)
  t1.start()
  return
  
def writeValueClicked(event):
  t2 = threading.Thread(target=processWriteValueClickedEvent)
  t2.start()
  return

def readValueClicked(event):
  t2 = threading.Thread(target=processReadValueClickedEvent)
  t2.start()
  return

# Use IPv4 address to speed-up calling resusts, because OVL does not use IPv6.
defaultUrl = "http://127.0.0.1:4880"

ovl = Ovl(defaultUrl)

window=tk.Tk()
window.title('ogamma Visual Logger for OPC API example client')
window.geometry("700x650+20+20")

mainFrame = ttk.Frame(window, padding="25 25 25 25")
mainFrame.grid(column=0, row=0, sticky="N, W, E, S")
window.columnconfigure(0, weight=1)
window.rowconfigure(0, weight=1)

# Row 1 - URL and credentials
urlVar = tk.StringVar(window, defaultUrl)
urlLabel = tk.Label(mainFrame, text='URL')
urlLabel.grid(column=1, row=1)
urlField = tk.Entry(mainFrame, textvariable=urlVar)
urlField.grid(column=2, row=1)

userVar = tk.StringVar(window, "admin")
userLabel = tk.Label(mainFrame, text='User name')
userLabel.grid(column=3, row=1)
userField = tk.Entry(mainFrame, textvariable=userVar)
userField.grid(column=4, row=1)

passwordVar = tk.StringVar(window, "password")
passwordLabel = tk.Label(mainFrame, text='Password')
passwordLabel.grid(column=5, row=1)
passwordField = tk.Entry(mainFrame, textvariable=passwordVar, show="*")
passwordField.grid(column=6, row=1)

# Row 2 - Variable id, current value and value to write
variableIdVar = tk.StringVar(window, "1")
variableIdLabel = tk.Label(mainFrame, text='Variable Id')
variableIdLabel.grid(column=1, row=2)
variableIdField = tk.Entry(mainFrame, textvariable=variableIdVar)
variableIdField.grid(column=2, row=2)

currentValueVar = tk.StringVar(window, "")
currentValueLabel = tk.Label(mainFrame, text='Read value')
currentValueLabel.grid(column=3, row=2)
currentValueField = tk.Entry(mainFrame, textvariable=currentValueVar)
currentValueField.grid(column=4, row=2)

valueToWriteVar = tk.StringVar(window, "0")
valueToWriteLabel = tk.Label(mainFrame, text='Value to write')
valueToWriteLabel.grid(column=5, row=2)
valueToWriteField = tk.Entry(mainFrame, textvariable=valueToWriteVar)
valueToWriteField.grid(column=6, row=2)

# Row 3 - buttons
getStatusBtn = ttk.Button(mainFrame, text='Get status')
getStatusBtn.grid(column=1, row=3, columnspan=2)
getStatusBtn.bind('<Button-1>', getStatusClicked)

readValueBtn = ttk.Button(mainFrame, text='Read value')
readValueBtn.grid(column=3, row=3, columnspan=2)
readValueBtn.bind('<Button-1>', readValueClicked)

writeValueBtn = ttk.Button(mainFrame, text='Write value')
writeValueBtn.grid(column=5, row=3, columnspan=2)
writeValueBtn.bind('<Button-1>', writeValueClicked)

# Row 4 - logging text field
lb=tk.Text(mainFrame)
lb.grid(column=1, row=4, columnspan=6, sticky="N, W, E, S")

mainFrame.pack()
window.mainloop()
